#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#ifdef UNIX
#include <malloc.h>
#endif
#include "fits.h"


/* program reads in a 2d fits image, allocates a 2d array for it
   and then returns the image and the size of the array */

struct fits_file rfits(char *filename)
     
{
  struct fits_file fits;
  int count=0,i,j=0,k=0;
  int naxis,naxis1,naxis2,hsize,place;
  char param[9],*header,*old;
  char *pt;
  char *rest;
  float fy;
  short sy;
  long int ly;
  double wpc,b0,bscale;
  int bitpix;
  int more = 1;
  char title[80];
  float *y;
  int *size;
  FILE *file;


  if((file = fopen(filename,"r"))==NULL){
    strcat(filename,".fit");
    if((file = fopen(filename,"r"))==NULL){
      fprintf(stderr,"Cannot find file name %s\n",filename);
      fits.naxis=0;
      return(fits);
      }
  }

  /*read in the FITS header in 2880 blocks looking for 
   the important key words*/
   
  hsize = 0;
  header = (char *) malloc(2880*sizeof(char));
  count = 0;
  while(more){       
    header = (char *) realloc(header,count+2880*sizeof(char));
    fread(&header[count],2880,1,file);
    pt = &header[count];
    count += 2880;	
    for(place=0;place < 2880; pt+=80,place+=80){
      sscanf(pt,"%8s",param);
      rest = pt + 10; 
      if(strcmp(param,"BZERO")==0) { /* zero offset for integer data */
	sscanf(rest," %lf ",&b0);
      }
      if(strcmp(param,"BSCALE")==0) { /*scale for integer data */
	sscanf(rest," %lf ",&bscale);
      }
      if(strcmp(param,"BITPIX")==0) { /* bits per pixel (negative for
					 ieee floats) */
	sscanf(rest," %d ",&bitpix);
      }
      if(strcmp(param,"NAXIS")==0) { /* nubmer of axis */
	sscanf(rest,"%d",&naxis);
        fits.naxis = naxis;
      }
      if(strcmp(param,"NAXIS1")==0) { /* size of axis 1 */
	sscanf(rest," %d ",&naxis1);
      }
      if(strcmp(param,"NAXIS2")==0) { /* size of axis 2 */
	sscanf(rest," %d ",&naxis2);
      }      
      if(strcmp(param,"END") == 0) {
        more = 0;
        break;
      }
    }
    hsize ++;
  }
  /* right now Im lazy, we'll deal with n dimentional data in a little while */
  /* just 2D data right now */
  size = (int *) malloc (naxis*sizeof(int));
  size[0]=naxis1;
  if(naxis == 1) {
    naxis2 = 1;
    size[1]=1;
  }
  if(naxis > 1 ) size[1]=naxis2;

  
  /* next make a 2d array of size naxis2 by naxis1 */
  
  y = (float *) malloc(naxis1*naxis2*sizeof(float *));
  if(!y) 
    fprintf(stderr,"allocation failure 1 in matrix()");
  /* then read in the nubers according to bitpix (16 = short 32 = long
     and - = float */

  if(bitpix > 0){
    if(bitpix==16){
      for(k=0;k<naxis2;k++){
	for(j=0;j<naxis1;j++){
	  fread(&sy,sizeof(short),1,file);
	  y[k*naxis1+j]=sy*bscale+b0;
	}
      }
    }
    else {
      for(k=0;k<naxis2;k++){
	for(j=0;j<naxis1;j++){
	  fread(&ly,sizeof(long int),1,file);
	  y[k*naxis1+j]=ly*bscale+b0;
	}
      }
    }
  }
  else {
    for(k=0;k<naxis2;k++){
      for(j=0;j<naxis1;j++){
	fread(&fy,sizeof(float),1,file);
	y[k*naxis1+j]=fy;
      }
    }
  }
  
  /* return the size of the array in x and y and the array itself */
  
  fits.size=size;
  fits.header=header;
  fits.data=y;
  fclose(file);
  return(fits);
}
  
      
